/*
 * scull.c 
 *
 * To show fasync asynchronous readers in driver module
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/proc_fs.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <linux/errno.h>

#include <linux/types.h>
#include <linux/fs.h>
#include <linux/fcntl.h>

#include <asm/uaccess.h>

MODULE_LICENSE("GPL");

/* function prototypes */
int scull_fasync (int fd, struct file *filp, int mode);
int scull_open (struct inode *inode, struct file *file); 
int scull_release (struct inode *inode, struct file *file); 
int init_module (void); 
void cleanup_module (void); 

typedef struct Scull_Pipe {
    struct fasync_struct *async_queue; /* asynchronous readers */
} Scull_Pipe;

Scull_Pipe *dev;
char scull_data[]="Linux is really funky!";

/* open module */
int scull_open (struct inode *inode, struct file *file) 
{
	/* for compiler warning */
	inode=inode;
	file=file;

	printk (KERN_WARNING"\nscull Data generator opened\n");
	return 0;
}

/* release module */
int scull_release (struct inode *inode, struct file *filp) 
{
	printk (KERN_WARNING"\nscull Data generator released\n");

	/* remove this file from asynchronouls notified filp's */
	if (dev->async_queue)
	  scull_fasync (-1, filp, 0);

	return 0;

	/* to prevent compiler warning */
	inode=inode;
	filp=filp;
}

/* read module */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos) 
{
	ssize_t result=0;

	printk (KERN_WARNING"\nscull Data Generator, reading %d bytes\n", count); 
	if (copy_to_user (buffer, scull_data, sizeof(scull_data)-1)) {
		result = -EFAULT;
		return result;
	}
	else 
		printk (KERN_WARNING"\nscull Data Generator, read %d bytes\n", count); 

	if (dev->async_queue)
		kill_fasync (&dev->async_queue, SIGIO, POLL_IN);
	printk (KERN_WARNING"\nkill_async called\n"); 

	return result;

	/* to prevent compiler warning */
	filp=filp;
	pos=pos;
}

int scull_fasync (int fd, struct file *filp, int mode)
{
    return fasync_helper (fd, filp, mode, &dev->async_queue);
}

struct file_operations scull_fops = {
	open:			scull_open,
	release:	scull_release,
  read:			scull_read,
	fasync:   scull_fasync,
};

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
  SET_MODULE_OWNER(&scull_fops);
	dev = kmalloc(sizeof(Scull_Pipe), GFP_KERNEL);
  if (dev == NULL)
  	return -ENOMEM;
  memset(dev, 0, sizeof(Scull_Pipe));

	printk (KERN_WARNING"\nscull Data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
	
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
  else {
		printk (KERN_WARNING "scull cleaned up :)");
	}

	kfree (dev);

}
