/*
 * ioctl.c 
 *
 * Code to test ioctl commands 
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/ioctl.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

#include "scull.h"

MODULE_LICENSE("GPL");

/* prototype declarations */
int scull_ioctl (struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
int scull_open (struct inode *inode, struct file *file); 
int scull_release (struct inode *inode, struct file *file); 
int init_module (void);
void cleanup_module (void); 

/* ioctl module */
int scull_ioctl (struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg) 
{
	int ret = 0;

	switch (cmd) {
		
			case SCULL_ONE:
				printk (KERN_INFO "SCULL_ONE called\n");
				break;

			case SCULL_TWO:
				printk (KERN_INFO "SCULL_TWO called\n");
				break;

			default:
				break;
	}
	return ret;

	/* for compiler warning */
	inode = inode;
	filp = filp;
	arg = arg;
}

/* open module */
int scull_open (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull Data generator opened\n");
	return 0;

	/* for compiler warning */
	inode = inode;
	file = file;
}

/* release module */
int scull_release (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull Data generator released\n");
	return 0;

	/* for compiler warning */
	inode = inode;
	file = file;
}

struct file_operations scull_fops = {
	ioctl: scull_ioctl,
	open:	scull_open,
	release:	scull_release,
};

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
	printk (KERN_WARNING"\nscull Data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
	
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
  else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
