/*
 * scull.c 
 *
 * To show allocation of memory using kmalloc and deallocations using kfree
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/slab.h>

MODULE_LICENSE("GPL");

/* prototype declarations */
int init_module (void);
void cleanup_module (void); 
 
unsigned long *buff = NULL;

/* init module */
int init_module (void) 
{
	int count = 32;

	buff = kmalloc ( sizeof(char) * count, GFP_KERNEL);
	if (!buff) {
		printk (KERN_INFO "init_module: buff: kmalloc failed\n");
		return -ENOMEM;
	}
	printk (KERN_INFO "init_module: allocated memory for buff\n");

	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	if (buff)
		kfree (buff);

	printk ("cleanup_module: cleaned up buff\n");
}
