/*
 * read.c 
 *
 * Read operation from kernel space (hardware) to user space. 
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

MODULE_LICENSE("GPL");

/* prototype declarations */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos); 
int scull_open (struct inode *inode, struct file *file); 
int scull_release (struct inode *inode, struct file *file); 
int init_module (void);
void cleanup_module (void); 

char scull_data[]="Linux is really funky!";

/* read module */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos) 
{
	ssize_t result=0;
	printk (KERN_WARNING"\nscull data generator, reading %d bytes\n", count); 
	if (copy_to_user (buffer, scull_data, sizeof(scull_data)-1)) {
		result = -EFAULT;
		return result;
	}

	/*
	 * unsigned long copy_to_user (void *to, const void *from,
	 *     unsigned long count);
	 */
	 
	else 
		printk (KERN_WARNING"\nscull data generator, read %d bytes\n", count); 
	return result;

	/* to prevent compiler warnings */
	filp = filp;
	pos = pos;
	count = count;
}

/* open module */
int scull_open (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull data generator opened\n");
	return 0;

	/* to prevent compiler warnings */
	file = file;
	inode = inode;
}

/* release module */
int scull_release (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull data generator released\n");
	return 0;

	/* to prevent compiler warnings */
	file = file;
	inode = inode;
}

struct file_operations scull_fops = {
	read:	scull_read,
	open:	scull_open,
	release:	scull_release,
};

/*
 * struct file_operations uses the tagged structure initialization syntax.
 *
 * Commonly used functions are:
 * 
 * struct file_operations scull_fops = {
 *   llseek: scull_llseek;
 *   read: scull_read;
 *   write: scull_write;
 *   ioctl: scull_ioctl;
 *   open: scull_open;
 *   release: scull_release;
 * };
 */

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
	printk (KERN_WARNING"\nscull data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	/*
	 * int register_chrdev (unsigned int major, const char *name,
	 *   struct file_operations *fops);
	 */
	
	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
	
  /* 
	 * int unregister_chrdev (unsigned int major, const char *name);
	 */
	
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
  else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
