/*
 * readminor.c 
 *
 * Read operation from kernel space (hardware) to user space from the 
 * appropriate minor device number.
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

#define MAX_MESG 30 

MODULE_LICENSE("GPL");

/* prototype declarations */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos); 
int scull_open (struct inode *inode, struct file *file); 
int scull_release (struct inode *inode, struct file *file); 
int init_module (void);
void cleanup_module (void); 

char scull_data1[]="hohoho for minor 0";
char scull_data2[]="heyhey for minor 1";

/* read module */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos) 
{
	ssize_t result=0;
	printk (KERN_WARNING"\nscull Data Generator, reading %d bytes\n", count);
	if (copy_to_user (buffer, filp->private_data, MAX_MESG)) {
		result = -EFAULT;
		return result;
	}
	else 
		printk (KERN_WARNING"\nscull_data Generator, read %d bytes\n", count); 
	return result;

	/* to prevent compiler warning */
	pos = pos;
}

/* open module */
int scull_open (struct inode *inode, struct file *filp) 
{
	printk (KERN_WARNING"\nscull Data generator opened\n");
	int num = MINOR (inode->i_rdev);
	printk (KERN_WARNING"\n num in scull_open = %d", num);
	if (num==0)  
		filp->private_data = scull_data1; /* assign scull_data1 for minor device 0 */
	else
		filp->private_data = scull_data2; /* assign scull_data2 for minor device 1 */ 
	return 0;

	/* to prevent compiler warning */
	inode = inode;
}

/* release module */
int scull_release (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull Data generator released\n");
	return 0;

	/* to prevent compiler warning */
	inode = inode;
	file = file;
}

struct file_operations scull_fops = {
	read:	scull_read,
	open:	scull_open,
	release:	scull_release,
};

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
	printk (KERN_WARNING"\nscull Data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
  
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
  else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
