/*
 * register.c 
 *
 * Registers a device with a major number and unregisters it on cleanup.
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */

#include <linux/module.h>
#include <linux/config.h>
#include <linux/init.h>

#include <linux/fs.h>

MODULE_LICENSE("GPL");

/* prototype declarations */
int init_module (void);
void cleanup_module (void); 

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	struct file_operations scull_fops;
	
	result = register_chrdev (scull_major, "scull", &scull_fops);

	/*
	 * int register_chrdev (unsigned int major, const char *name,
	 *   struct file_operations *fops);
	 */
	
	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major=result;
	printk ("scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");

  /* 
	 * int unregister_chrdev (unsigned int major, const char *name);
	 */
	
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
  else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
