/*
 * write.c 
 *
 * Write operation from user space to kernel space (hardware)
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

MODULE_LICENSE("GPL");

/* prototype declarations */
ssize_t scull_write (struct file *fp, const char *buf, size_t count, loff_t *f_pos); 
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos); 
int scull_open (struct inode *inode, struct file *file); 
int scull_release (struct inode *inode, struct file *file); 
int init_module (void);
void cleanup_module (void); 

char scull_data[128]="\0";

/* write module */
ssize_t scull_write (struct file *fp, const char *buf, size_t count, loff_t *f_pos) 
{
	ssize_t ret=0;
	printk (KERN_WARNING"\n Writing %d bytes\n", count);
	if (count>127) return -ENOMEM;
	if (count<0) return -EINVAL;
	if (copy_from_user(scull_data, buf, count)) {
		ret = -EFAULT; 
		return ret;
  }
	
	else {	
	  scull_data[127]='\0';
		printk (KERN_WARNING"\nReceived: %s\n",scull_data);
		ret = count;
		return ret;
	}

	/* to prevent compiler warning */
	f_pos = f_pos;
	fp = fp;
}

/* read module */
ssize_t scull_read (struct file *filp, char *buffer, size_t count, loff_t *pos) 
{
	ssize_t result=0;
	printk (KERN_WARNING"\nscull Data Generator, reading %d bytes\n", count); 
	if (copy_to_user (buffer, scull_data, sizeof(scull_data)-1)) {
		result = -EFAULT;
		return result;
	}

	else 
		printk (KERN_WARNING"\nscull Data Generator, read %d bytes\n", count); 
	return result;

	/* to prevent compiler warning */
	pos = pos;
	filp = filp;
}

/* open module */
int scull_open (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull Data generator opened\n");
	return 0;

	/* to prevent compiler warning */
	inode = inode;
	file = file;
}

/* release module */
int scull_release (struct inode *inode, struct file *file) 
{
	printk (KERN_WARNING"\nscull Data generator released\n");
	return 0;

	/* to prevent compiler warning */
	inode = inode;
	file = file;
}

struct file_operations scull_fops = {
	read:	scull_read,
	write: scull_write,
	open:	scull_open,
	release:	scull_release,
};

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
	printk (KERN_WARNING"\nscull Data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
	
	else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
