/*
 * writeminor.c 
 *
 * Write operation from user space to kernel space (hardware) to a specific
 * minor device number
 *
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 *
 */
 
#include <linux/module.h>
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/fs.h>

#include <asm/uaccess.h>

MODULE_LICENSE("GPL");

char scull_data1[128]="\0";
char scull_data2[128]="\0";

/* prototype declarations */
ssize_t scull_write (struct file *fp, const char *buf, size_t count, loff_t *f_pos);
int scull_open (struct inode *inode, struct file *filp);
int scull_release (struct inode *inode, struct file *filp);
int init_module (void);
void cleanup_module (void); 

/* write module */
ssize_t scull_write (struct file *fp, const char *buf, size_t count, loff_t *f_pos) 
{
	ssize_t ret=0;
	int num = (int)fp->private_data;
	printk (KERN_WARNING"Writing %d bytes\n", count);
	if (count>127) return -ENOMEM;
	if (count<0) return -EINVAL;
	if (num==0) {
		if (copy_from_user(scull_data1, buf, count)) {
			ret = -EFAULT; 
			return ret;
  	}
	
		else {	
	  	scull_data1[127]='\0';
			printk (KERN_WARNING"Received: scull_data1 = %s\n",scull_data1);
			ret = count;
			return ret;
		}
	}
	else {
		if (copy_from_user(scull_data2, buf, count)) {
			ret = -EFAULT; 
			return ret;
		}
	
		else {	
	  	scull_data2[127]='\0';
			printk (KERN_WARNING"Received: scull_data2 = %s\n",scull_data2);
			ret = count;
			return ret;
		}
	}
	
	/* to prevent compiler warning */
	f_pos = f_pos;
		
}


/* open module */
int scull_open (struct inode *inode, struct file *filp) 
{
	int num = MINOR (inode -> i_rdev);
	printk (KERN_WARNING"\nscull Data generator opened\n");
	if (num==0) 
		filp->private_data = (void *)0;
	else
		filp->private_data = (void *)1;
	return 0;

	/* to prevent compiler warning */
	inode = inode;
}

/* release module */
int scull_release (struct inode *inode, struct file *filp) 
{
	printk (KERN_WARNING"\nscull Data generator released\n");
	return 0;

	/* to prevent compiler warning */
	inode = inode;
	filp = filp;
}

struct file_operations scull_fops = {
	write: scull_write,
	open:	scull_open,
	release:	scull_release,
};

/* init module */
int init_module (void) 
{
	int result;
	unsigned int scull_major=241;
	
	printk (KERN_WARNING"\nscull Data generator init module done\n");
	result = register_chrdev (scull_major, "scull", &scull_fops);

	if (result <0) {
		printk (KERN_WARNING "scull: can't get major %d\n", scull_major);
		return result;
	}

	if (scull_major==0) scull_major = result;
	printk (KERN_WARNING"scull_major = %d\n",scull_major);
	 
	return 0;
}

/* cleanup module */
void cleanup_module (void) 
{
	int n;
	n  = unregister_chrdev (241, "scull");
	if (n < 0 ) {
		printk (KERN_WARNING "cleanup: cannot unregister_chrdev %d",n);
	}
	
	else {
		printk (KERN_WARNING "scull cleaned up :)");
	}
}
